import {executeTests, formatResult} from './web_exploit_detector';
import Logger from './logger';

import Promise  from 'bluebird';
import minimist from 'minimist';

const fs = Promise.promisifyAll(require('graceful-fs'));

const argv = minimist(
  process.argv.slice(2),
  {
    boolean: true,
    default: {
      'debug': false,
      'exclude-tags': undefined,
      'help': false,
      'include-deprecated': false,
      'parallel': true,
      'tags': undefined,
      'verbose': false,
      'webroot': undefined,
    },
  }
);

const logger = new Logger(argv);

function printUsage() {
  logger.log(3, `Usage: node ${process.argv[1]} --webroot=<web_root>`);
  logger.log(3, '  --debug: sets the debug flag: enables the "debug" rules.');
  logger.log(3, '  --exclude-tags=<list>: excludes all rules that include one or more of the comma-separated tags in <list>.');
  logger.log(3, '  --help: shows this message');
  logger.log(3, '  --include-deprecated: includes rulesmarked as deprecated.');
  logger.log(3, '  --no-parallel: disables parallel processing if set. While parallel processing is technically faster, it has a higher resource usage when running.');
  logger.log(3, '  --output=<file>: writes all rules with matched file paths (if any) in JSON format to <file>.');
  logger.log(3, '  --tags=<list>: only runs rules matching the comma-separated <list> of tags.');
  logger.log(3, '  --verbose: enables verbose log output.');
  logger.log(3, '  --webroot=<web_root>: sets the base directory that will be searched recursively to <web_root>.');
  logger.log(3, '\n\nExamples: ');
  logger.log(3, '  - Search the /var/www/html directory structure: -');
  logger.log(3, `    node ${process.argv[1]} --webroot=/var/www/html\n`);
  logger.log(3, '  - Search for actual exploits only, not suspicious files: -');
  logger.log(3, `    node ${process.argv[1]} --webroot=/var/www/html --exclude-tags=suspicion\n`);
  logger.log(3, '  - Search using only WordPress-related rules: -');
  logger.log(3, `    node ${process.argv[1]} --webroot=/var/www/html --tags=wordpress\n`);
  logger.log(3, '  - Search using all rules except those tagged with "jquery" or "js": -');
  logger.log(3, `    node ${process.argv[1]} --webroot=/var/www/html --exclude-tags=jquery,js\n`);
}

export default function execute() {
  logger.log(3, 'Web Exploit Detector by Simon Pugnet ((c) 2017)\n');

  if (argv.help)
  {
    printUsage();
    process.exit(0);
  }

  if (!argv.webroot)
  {
    logger.log(1, 'ERROR: --webroot command line argument must be specified\n');
    printUsage();
    process.exit(1);
  }

  executeTests({
    debug:              argv.debug,
    exclude_tags:       argv['exclude-tags'],
    include_deprecated: argv['include-deprecated'],
    parallel:           argv.parallel,
    tags:               argv.tags,
    verbose:            argv.verbose,
    webroot:            argv.webroot,

    logger: logger.log,
  })

    // Output results JSON to file if required
    .then(res => {
      if (argv.output && typeof argv.output !== 'boolean' && argv.output.toString().length > 0)
      {
        const resultsWithMatches = res.filter(x => x.matches.length > 0);
        if (resultsWithMatches.length === 0)
        {
          logger.log(4, 'No matches found, so nothing to output to file');
          return res;
        }
        logger.log(4, `Outputting results to ${argv.output}...`);
        return fs.writeFileAsync(
          argv.output,
          JSON.stringify(
            resultsWithMatches.map(x => ({
              rule: {
                id:          x.rule.id,
                name:        x.rule.name,
                description: x.rule.description,
                tags:        x.rule.tags,
                deprecated:  x.rule.deprecated,
                urls:        x.rule.urls,
              },
              matches: x.matches,
            })),
            null,
            '\t'
          )
        )
          .then(() => {
            return res;
          })
          .catch(err => {
            logger.log(1, `ERROR: unable to write results to file: ${err.toString()}`);
            return res;
          });
      }
      return res;
    })

    // Write results to console
    .then(res =>
      logger.log(3, 
        'Processing complete, results: -\n\n' +
        res
          .map(formatResult)
          .filter(x => x !== null)
          .join('\n')
      )
    );
}
